/*
 * imageLoad.h - pro paraleln vpoet dopen a zptn expanzivn DTWT po segmentech, separabilnm pstupem algoritmem segDTWT2D
 * Autor: Zdenk Pra, UTKO, FEKT, VUT v Brn, Purkyova 118, Brno
 * Program vznikl za podpory internho grantu FEKT-J-10-8 - Optimln algoritmy paralelnho vpotu waveletov transformace rozshlch obrazovch dat
 * Extern knihovny: Intel Threading Building Blocks 3.0
 */

#pragma once
#ifndef imageLoad_H
#define imageLoad_H
#include <iostream>
#include <fstream>
#include "segDTWT.h"
#define ALIGN_TO_BYTES 16

typedef unsigned char BYTE;
typedef unsigned short WORD;

#pragma pack(1)
typedef struct{
BYTE	IDLength;		// velikost obrazovho identifiktoru
BYTE	ColorMapType;	// typ barevn mapy
BYTE	Imagetype;		// typ obrzku
WORD	CMapStart;		// potek barevn palety
WORD	CMapLength;		// dlka barevn palety
BYTE	CMapDepth;		// bitov hloubka poloek barevn palety
WORD	XOffset;		// X-ov souadnice potku obrzku
WORD	YOffset;		// Y-ov souadnice potku obrzku
WORD	Width; 			// ka obrzku uveden v pixelech
WORD	Height; 		// vka obrzku uveden v pixelech
BYTE	PixelDepth; 	// poet bit na jeden pixel (bitov hloubka)
BYTE	ImageDescriptor;//	popisova obrzku
} TGAheader;

#pragma pack()


using namespace std;


/*!\~czech
Funkce nate nekomprimovan obrzek ve formtu tga s nzvem name, jeho okraje roz o r pixel, a vrt ukazatel na zatek ji rozench dat. Podle zvolenho zpsobu prodluovn se provede rozen okraj.
\param[in] *name -	nzev tga obrzku
\param[in] r	 - hodnota \f$r =(2^J-1)(m-1) \f$ kde \f$J\f$ je stupe dekompozice a \f$m\f$ je dlka filtr.
\param[out] *widthStep - ukazatel pro uloen hodnoty vzdlenosti mezi jednotlivmi dky pixel obrazu v pamti v bytech
\param[out] *size - rozmry jet neprodlouenho obrazu
\return Ukazatel na zatek ji rozench dat.

\~english
The function loads uncompressed tga image name, the image is extended by r pixels in all directions and the pointer to the beginning of the extended image is returned. Values of the extended pixels are chosen according to the chosen boundary treatment method.
\param[in] *name -	tga image
\param[in] r	 - value of \f$r =(2^J-1)(m-1) \f$, where \f$J\f$ is the depth of decomposition and \f$m\f$ is filter length
\param[out] *widthStep - pointer to the value defining distance between two consecutive rows of the image in memory in bytes.
\param[out] *size - pointer to the dimensions of the original image.
\return pointer to the beginning of the extended image
*/

float* loadWrapAlignTGA(const char* name,int r, int *widthStep, Size* size);

/*!\~czech
Funkce ulo obraz v poli definovanho ukazatelem image, rozmr size jako nakomprimovan tga obraz s nzvem name.
\param[in] *name -	nzev tga obrzku pro uloen, mus obsahovat .tga
\param[in] *image	 - ukazatel na obrzek k uloen
\param[in] widthStep - vzdlenost mezi jednotlivmi dky pixel obrazu v pamti v bytech
\param[in] size - rozmry obrazu
\return int <0 jako indikaci chyby.

\~english
The function saves the image define d by the pointer image and dimensions size as a tga file name.
\param[in] *name -	fine nalem. Should contain extension .tga
\param[in] *image	 - pointer to the beginning of the image
\param[in] widthStep - distance between two consecutive rows of the image in memory in bytes
\param[in] size - image dimensions
\return int <0 indicates an error.
*/
int saveTGA(const char* name,float* image, int widthStep, Size size);


void wrapBorder_sp0_32f_C1(float* o_data, int o_widthStep,int r,Size size);

void wrapBorder_symh_32f_C1(float* o_data, int o_widthStep,int r,Size size);

void wrapBorder_symw_32f_C1(float* o_data, int o_widthStep,int r,Size size);


 #endif 