/*
 * segDTWT.h - paraleln vpoet dopen a zptn expanzivn DTWT po segmentech, separabilnm pstupem algoritmem segDTWT2D
 * Autor: Zdenk Pra, UTKO, FEKT, VUT v Brn, Purkyova 118, Brno
 * Program vznikl za podpory internho grantu FEKT-J-10-8 - Optimln algoritmy paralelnho vpotu waveletov transformace rozshlch obrazovch dat
 * Extern knihovny: Intel Threading Building Blocks 3.0
 */
/*!
\file segDTWT.h
 * Znaen jednotlivch subpsem:
 <pre>
 -------------------------------------- 
 |LL |HL3|      HL2|               HL1| 
 |--------         |                  |
 |LH3|HH3|         |                  |
 |------------------                  |
 |    LH2|      HH2|                  |
 |       |         |                  |
 |       |         |                  |
 |-------------------------------------
 |              LH1|               HH1|
 |                 |                  |
 |                 |                  |
 |                 |                  |
 |                 |                  |
 |                 |                  |
 --------------------------------------
  </pre>
 *
 * Program pracuje s:
 *        - s volitelnou hloubkou vlnkov transformace
 *        - libovolnmi filtry 
 *        - vbr roziovn okraj
 *

 */


#ifndef segDTWT_H
#define segDTWT_H

// konstanty
#define ALIGN_TO_BYTES 16
#define SUBBAND_COUNT 3
#define SUBSAMPLING 1

//MAKRA
#define pow2(x) ((1)<<(x))  // makro pro mocninu 2 pomoc bitovho posunu
#define r(k,m) (pow2(k)-(1))*((m)-(1))
#define max(x,y) x>y?x:y
#define min(x,y) x<y?x:y
#define odd(x) (x%2)!=0
#define even(x) (x%2)==0

typedef enum {
ZEROS_PADDING = 0,
CONST_PADDING = 1,
SYMETRIC_HP = 2,
SYMETRIC_WP = 3
} wrapType;

typedef struct {
int x;
int y;
int width;
int height;
}Rect;

typedef enum {
RESULT_OK = 0,
RESULT_UNKNOWN_ERROR = -1
}WRESULT;

typedef struct {
int width;
int height;
}Size;

#include "cppTypes.h"
#include "memoryAllocation.h"
#include "imageLoad.h"
#include "tbb/tbb.h"
#include "rangeSegDTWTauto.h"
#include "bodySegDTWT2D.h"
#include "bodySegIDTWT2D.h"


/*
* Typ prodluovn
*/
extern wrapType extensionType;

/*!
* Velikost segment, kter ji nebudou dle dleny
 0,0 znamen, e bude piazeno 2*r(J)
*/
extern Size segSize;

/*
* Vchoz soubor pro natn koeficient filtr
*/
extern char* waveletFile;


/*

*/
// subbandy budou v poad HL1,HH1,LH1,HL2,HH2,LH2 .... LL
// vechny se mus pedem alokovat na zarovnan boundary rozmry  s(k+1) = floor((sk+m-1)/2) 
//float* subband[levels*SUBBAND_COUNT+1];
//int widthSteps[levels+1];
// subband rozmry: n(s,m,j) = floor( 2^-j)*s + (1-2^(-j)*(m-1))

/*!
\~czech
 Funkce segDTWTfwd_32f_C1 pro paraleln dopednou vlnkovou transformaci. Pracuje s jednokanlovm obrazem (C1) datovch typ float
 
\param[in] i_data			ukazatel na vstupn obraz
\param[in] i_widthStep		vzdlenost mezi dky obrzku v pamti v bytech
\param[in,out] subbands[]		pole ukazatel na vstupn subpsma. Ukazatele na subpsma jsou za sebou uloeny v poad subbands[0]=HL1,[1]=HH1,[2]=LH1,[3]=HL2,[4]=HH2,[5]=LH2 .... [last]=LL. Pam mus bt pedem alokovna a rozmry subpsem jsou \f[ n_\mathrm{x}(s,m,j) = \mathrm{floor}( 2^{-j}s_\mathrm{x} + (1-2^{-j}(m-1)) \f] a 
							\f[ n_\mathrm{y}(s,m,j) = \mathrm{floor}( 2^{-j}s_\mathrm{y} + (1-2^{-j}(m-1)), \f] piem
							- \f$ s,s_x,s_y \f$ jsou rozmry vstupnho obrazu,
							- \f$ j \f$ stupe dekompozice,
							- \f$ m \f$ dlka pouitch filtr.
							

\param[in,out] widthSteps[]	pole vzdlenost mezi dky subpsem v pamti v bytech	
\param[in] levels			stupe dekompozice
\param[in] size				rozmry vstupnho obrazu bez prodlouen
\param[in] w				pouit vlnkov filtry
\return Chybov hlen	WRESULT 		
\sa segDTWTinv_32f_C1

\~english
Function segDTWTfwd_32f_C1 for parallel forward discrete wavelet transform. Works with one channel image (C1) of floats.
\param[in] i_data - pointer to the beginning of the image
\param[in] i_widthStep - distance between two consecutive rows of the image in memory in bytes
\param[in,out] *subbands - array of pointers to the output subbands. The pointers are stored in the following order: subbands[0]=HL1,[1]=HH1,[2]=LH1,[3]=HL2,[4]=HH2,[5]=LH2 .... [last]=LL. The memory need to be allocader beforehand as following: \f[ n_\mathrm{x}(s,m,j) = \mathrm{floor}( 2^{-j}s_\mathrm{x} + (1-2^{-j}(m-1)) \f] and 
							\f[ n_\mathrm{y}(s,m,j) = \mathrm{floor}( 2^{-j}s_\mathrm{y} + (1-2^{-j}(m-1)), \f], where
							- \f$ s,s_x,s_y \f$ dimensions of the input image,
							- \f$ j \f$ depth of decomposition,
							- \f$ m \f$ the length of filters.
\param[in,out] widthSteps[] - array of the distances between two consecutive rows in bytes in EACH individual subband 
\param[in] levels - depth of decomnposition
\param[in] size - dimensions of the input image
\param[in] w - object defining wavelet filters
\return Error code WRESULT
\sa segDTWTinv_32f_C1
*/

WRESULT segDTWTfwd_32f_C1(float* i_data, int i_widthStep, float* subbands[], int widthSteps[], int levels, Size size, separableWavelet* w);





/*!
\~czech
 Funkce segDTWTinv_32f_C1 pro paraleln zptnou vlnkovou transformaci.
\param[in,out] subbands[]	-	pole ukazatel na subpsma. 	
\param[in,out] widthSteps[]	-pole vzdlenost mezi dky subpsem v pamti v bytech	
\param[in] o_data			-ukazatel na vstupn obraz
\param[in] o_widthStep		-vzdlenost mezi dky obrzku v pamti v bytech
\param[in] levels			-stupe dekompozice
\param[in] size				-rozmry vstupnho obrazu bez prodlouen
\param[in] w				-pouit vlnkov filtry
\return Chybov hlen	WRESULT 		
\sa segDTWTfwd_32f_C1

\~english
Function segDTWTinv_32f_C1 for parallel inverse discrete wavelet transform. 
\param[in,out] subbands[]	-	array of poinert to the individual subbands	
\param[in,out] widthSteps[]	-array of the distances between two consecutive rows in bytes in EACH individual subband 
\param[in] o_data			-pointer to the output image
\param[in] o_widthStep		-distance between two consecutive rows of the image in memory in bytes
\param[in] levels			-depth of decomposition
\param[in] size				-dimensions of the output image
\param[in] w				-object defining wavelet filtes
\return Chybov hlen	WRESULT 		
\sa segDTWTfwd_32f_C1
*/

WRESULT segDTWTinv_32f_C1(float* subbands[],int widthSteps[], float* o_data, int o_widthStep, int levels, Size size, separableWavelet* w);


#endif