/*
 * bodySegDTWT2D.h - pro paraleln vpoet dopen a zptn expanzivn DTWT po segmentech, separabilnm pstupem algoritmem segDTWT2D
 * Autor: Zdenk Pra, UTKO, FEKT, VUT v Brn, Purkyova 118, Brno
 * Program vznikl za podpory internho grantu FEKT-J-10-8 - Optimln algoritmy paralelnho vpotu waveletov transformace rozshlch obrazovch dat
 * Extern knihovny: Intel Threading Building Blocks 3.0
 */

#ifndef bodySegDTWT2D_H
#define bodySegDTWT2D_H

#include "memoryAllocation.h"
#include "rangeSegDTWTauto.h"
#include "segDTWT.h"
#include <algorithm>
#include <math.h>
#include <string>
#include <iostream>
#include "cppTypes.h"


// size = s + 2*(m-1)
// vsledek bude dlky floor((size -( filter_size -1))/2)
void convolveRow(float* i, int size,float* o,float* filter,int filter_size);

// konvoluce filtrem kratm ne pro kter bylo prodluovno
// filter_size je krat filtr
// vsledek je dlky floor((size -( filter_size + zerosLeft + zerosRight -1))/2) (take stejn jako ml del)
void convolveRow(float* i, int size, float* o,float* filter,int filter_size,int zerosLeft,int zerosRight);


// size = s + 2*(m-1)
// vsledek bude dlky floor((size -( filter_size -1))/2)
void convolveCol(float* i, int size,int i_widthStep,float* o,int o_widthStep,float* filter,int filter_size);

// konvoluce sloupce filtrem kratm, ne pro kter bylo prodloueno, vstupnch koeficient by ale mlo bt stejn
void convolveCol(float* i, int size,int i_widthStep,float* o,int o_widthStep,float* filter,int filter_size,int zerosTop,int zerosBottom);




/*!
\~czech
Tda bodySegDTWT2D slou jako funkn tda pro definici tla cyklu parallel_reduce pro vpoet dopedn transformace
\~english
The class bodySegDTWT2D is function class defining loop body of parallel_reduce for forward transform
 */
class bodySegDTWT2D{
// buffer pro LL psmo
float* LL_buffer;
int LL_buffer_widthStep;
// buffer pro LHL psmo
float* HL_buffer;
int HL_buffer_widthStep;
// buffer pro HH psmo
float* HH_buffer;
int HH_buffer_widthStep;
// buffer pro LH psmo
float* LH_buffer;
int LH_buffer_widthStep;

// buffer pro dkov H psmo
float* H_buffer;
int H_buffer_widthStep;
// buffer pro dkov L psmo
float* L_buffer;
int L_buffer_widthStep;

// ukazatel na cel obrzek a jeho widthStep
float* i_data;
int i_widthStep;
// pole ukazatel na subbandy
float** o_data; 
// pole jejich widthStep
int* widthSteps;

// definice vlnky
separableWavelet* w;
// velikost buffer odvozen z teoreticky nejvetsiho bloku
Size maxSize;
// rove dekompozice
int level;

// maximln velikost bufferu
int maxWidthHalf;
int maxHeightHalf;


public :
	/*!
\~czech
Metoda je petenm opertoru kulat zvorky a definuje vlastn tlo paralelnho cyklu parallel_reduce
\param[in] &r - reference na objekt tdy blocked_range2d_segdtwt_segidtwt_auto<size_t> definujc aktuln st iteranho prostoru
\~english
The method is definines overload of the round brackets oparator and server as a body of the parallel loop  parallel_reduce
\param[in] &r - the reference to the object of the class blocked_range2d_segdtwt_segidtwt_auto<size_t> denoting the actual part of the iteration space
*/
	void operator()(const tbb::blocked_range2d_segdtwt_segidtwt_auto<size_t>& r);

	/*!
\~czech
Konstruktor
\param[in] *i_data		-	ukazatel na vstupn obraz
\param[in] i_widthStep	-	vzdlenost mezi dky obrzku v pamti v bytech
\param[in,out] *subbands[]	-   pole ukazatel na vstupn subpsma. Ukazatele na subpsma jsou za sebou uloeny v poad subbands[0]=HL1,[1]=HH1,[2]=LH1,[3]=HL2,[4]=HH2,[5]=LH2 .... [last]=LL.
\param[in] widthSteps[]	-   pole vzdlenost mezi dky subpsem v pamti v bytech	
\param[in] levels		-	stupe dekompozice
\param[in] size			-	rozmry vstupnho obrazu bez prodlouen
\param[in] *w			-	objekt definujc pouit vlnkov filtry
\sa separableWavelet
\~english
Constructor
\param[in] *i_data		-	poiter to the input image
\param[in] i_widthStep	-	the number of bytes between two conscutive rows
\param[in,out] *subbands[]	-   array holding ponters to the individual subbands. Order: subbands[0]=HL1,[1]=HH1,[2]=LH1,[3]=HL2,[4]=HH2,[5]=LH2 .... [last]=LL.
\param[in] widthSteps[]	-   array holding the distance between two consecutive rows in bytes in individual subbands	
\param[in] levels		-	the depth of decomposition
\param[in] size			-	dimensions of the input image
\param[in] *w			-	object defining wavelet filters
\sa separableWavelet
*/
	bodySegDTWT2D(float* i_data, int i_widthStep, float* subbands[],int widthSteps[],Size maxSize, separableWavelet* w,int level):
	i_data(i_data),i_widthStep(i_widthStep),o_data(subbands),widthSteps(widthSteps),maxSize(maxSize),w(w),level(level){
		H_buffer = NULL;
		L_buffer = NULL;
		LL_buffer = NULL;
		LH_buffer = NULL;
		HL_buffer = NULL;
		HH_buffer = NULL;
		maxWidthHalf = 0;
		maxHeightHalf = 0;
	}
/*!
\~czech
Splitting konstruktor. Pro poteby knihovny TBB.
\param[in] &s		-	reference na pvodn objekt  
\param[in] tbb::split	-	konstantn paramer, odliujc splitting konstruktor od koprovacho konstruktoru

\~english
Splitting construktor. For the sake of the TBB library.
\param[in] &s		-	reference to the original object 
\param[in] tbb::split	-	constant paramer,  to differ splitting constructor from the copy constructor
*/
	bodySegDTWT2D(bodySegDTWT2D& s,tbb::split):
	i_data(s.i_data),i_widthStep(s.i_widthStep),o_data(s.o_data),widthSteps(s.widthSteps),maxSize(s.maxSize),w(s.w),level(s.level){
		//	std::cout<< "Splitting konstruktor " << std::endl;
		H_buffer = NULL;
		L_buffer = NULL;
		LL_buffer = NULL;
		LH_buffer = NULL;
		HL_buffer = NULL;
		HH_buffer = NULL;
		maxWidthHalf = 0;
		maxHeightHalf = 0;
	}
	/*!
\~czech
Koprovac konstruktor.
\param[in] &s		-	reference na pvodn objekt  

\~english
Copy construktor. 
\param[in] &s		-	reference to the original object 
*/
	
	bodySegDTWT2D(const bodySegDTWT2D& s):
	i_data(s.i_data),i_widthStep(s.i_widthStep),o_data(s.o_data),widthSteps(s.widthSteps),maxSize(s.maxSize),w(s.w),level(s.level){
		H_buffer = NULL;
		L_buffer = NULL;
		LL_buffer = NULL;
		LH_buffer = NULL;
		HL_buffer = NULL;
		HH_buffer = NULL;
		maxWidthHalf = 0;
		maxHeightHalf = 0;
	}

/*!
\~czech
Destruktor

\~english
Destructor

*/
	~bodySegDTWT2D();

	/*!
\~czech
Funkce pro poteby paralelnho cyklu parallel_reduce

\~english
Function demanded by the parallel_reduce loop
*/
	void join(const bodySegDTWT2D& s){}
};

#endif