/*
 * bodySegIDTWT2D.h - pro paraleln vpoet dopen a zptn expanzivn DTWT po segmentech, separabilnm pstupem algoritmem segDTWT2D
 * Autor: Zdenk Pra, UTKO, FEKT, VUT v Brn, Purkyova 118, Brno
 * Program vznikl za podpory internho grantu FEKT-J-10-8 - Optimln algoritmy paralelnho vpotu waveletov transformace rozshlch obrazovch dat
 * Extern knihovny: Intel Threading Building Blocks 3.0
 */

#ifndef bodySegIDTWT_H
#define bodySegIDTWT_H
#include "segDTWT.h"
#include "cppTypes.h"

// size = s + 2*(m-1)
// vsledek bude dlky floor((size -( filter_size -1))/2)
void convolveIRow(float* i, int size,float* o,float* filter,int filter_size);

// konvoluce filtrem kratm ne pro kter bylo prodluovno
// filter_size je krat filtr
// vsledek je dlky floor((size -( filter_size + zerosLeft + zerosRight -1))/2) (take stejn jako ml del)
void convolveIRow(float* i, int size, float* o,float* filter,int filter_size,int zerosLeft,int zerosRight);

// size = s + 2*(m-1)
// vsledek bude dlky floor((size -( filter_size -1))/2)
void convolveICol(float* i, int size,int i_widthStep,float* o,int o_widthStep,float* filter,int filter_size);


// konvoluce sloupce filtrem kratm, ne pro kter bylo prodloueno, vstupnch koeficient by ale mlo bt stejn
void convolveICol(float* i, int size,int i_widthStep,float* o,int o_widthStep,float* filter,int filter_size,int zerosTop,int zerosBottom);

/*!
~czech
Tda bodySegDTWT2D slou jako funkn tda pro definici tla cyklu parallel_reduce pro vpoet dopedn transformace
~english
The class bodySegIDTWT2D is function class defining loop body of parallel_reduce for inverse transform
 */
class bodySegIDTWT2D{
	// buffer pro dkov H psmo
	float* row_buffer;
	int row_buffer_widthStep;
	// buffer pro dkov L psmo
	float* col_buffer;
	int col_buffer_widthStep;

	// buffer pro dkov L psmo
	float* third_buffer;
	int third_buffer_widthStep;

	// ukazatel na cel obrzek a jeho widthStep
	float* o_data;
	int o_widthStep;
	// pole ukazatel na subbandy
	float** subbands; 
	// pole jejich widthStep
	int* widthSteps;

	// definice vlnky
	separableWavelet* w;
	// velikost buffer odvozen z teoreticky nejvetsiho bloku
	Size maxSize;
	// rove dekompozice
	int level;

	// maximln velikost bufferu
	int maxWidth;
	int maxHeight;


public :
		/*!
\~czech
Metoda je petenm opertoru kulat zvorky a definuje vlastn tlo paralelnho cyklu parallel_reduce
\param[in] &r - reference na objekt tdy blocked_range2d_segdtwt_segidtwt_auto<size_t> definujc aktuln st iteranho prostoru
\~english
The method is definines overload of the round brackets oparator and server as a body of the parallel loop  parallel_reduce
\param[in] &r - the reference to the object of the class blocked_range2d_segdtwt_segidtwt_auto<size_t> denoting the actual part of the iteration space
*/
	void operator()(const tbb::blocked_range2d_segdtwt_segidtwt_auto<size_t>& r);
	/*!
\~czech
Konstruktor
\param[in,out] *subbands[]	-   pole ukazatel na vstupn subpsma. Ukazatele na subpsma jsou za sebou uloeny v poad subbands[0]=HL1,[1]=HH1,[2]=LH1,[3]=HL2,[4]=HH2,[5]=LH2 .... [last]=LL.
\param[in] widthSteps[]	-   pole vzdlenost mezi dky subpsem v pamti v bytech	
\param[in] *o_data		-	ukazatel na vstupn obraz
\param[in] o_widthStep	-	vzdlenost mezi dky obrzku v pamti v bytech
\param[in] size			-	rozmry vtupnho obrazu
\param[in] *w			-	objekt definujc pouit vlnkov filtry
\param[in] level		-	stupe dekompozice
\sa separableWavelet
\~english
Constructor
\param[in,out] *subbands[]	-   array holding ponters to the individual subbands. Order: subbands[0]=HL1,[1]=HH1,[2]=LH1,[3]=HL2,[4]=HH2,[5]=LH2 .... [last]=LL.
\param[in] widthSteps[]	-   array holding the distance between two consecutive rows in bytes in individual subbands	
\param[in] *o_data		-	poiter to the output image
\param[in] o_widthStep	-	the number of bytes between two conscutive rows
\param[in] size			-	dimensions of the output image
\param[in] *w			-	object defining wavelet filters
\param[in] level		-	the depth of decomposition
\sa separableWavelet
*/
	bodySegIDTWT2D( float* subbands[],int widthSteps[],float* o_data, int o_widthStep,Size maxSize, separableWavelet* w,int level):
	o_data(o_data),o_widthStep(o_widthStep),subbands(subbands),widthSteps(widthSteps),maxSize(maxSize),w(w),level(level){
		// std::cout<< "Konstruktor " << std::endl;
		third_buffer=NULL;	
		row_buffer=NULL;	
		col_buffer=NULL;
		maxWidth = 0;
		maxHeight = 0;
	}

	/*!
\~czech
Splitting konstruktor. Pro poteby knihovny TBB.
\param[in] &s		-	reference na pvodn objekt  
\param[in] tbb::split	-	konstantn paramer, odliujc splitting konstruktor od koprovacho konstruktoru

\~english
Splitting construktor. For the sake of the TBB library.
\param[in] &s		-	reference to the original object 
\param[in] tbb::split	-	constant paramer,  to differ splitting constructor from the copy constructor
*/
	bodySegIDTWT2D(bodySegIDTWT2D& s,tbb::split):
	o_data(s.o_data),o_widthStep(s.o_widthStep),subbands(s.subbands),widthSteps(s.widthSteps),maxSize(s.maxSize),w(s.w),level(s.level){
		// std::cout<< "Splitting konstruktor " << std::endl;
		third_buffer=NULL;	
		row_buffer=NULL;	
		col_buffer=NULL;
		maxWidth = 0;
		maxHeight = 0;


	}

		/*!
\~czech
Koprovac konstruktor.
\param[in] &s		-	reference na pvodn objekt  

\~english
Copy construktor. 
\param[in] &s		-	reference to the original object 
*/
	bodySegIDTWT2D(const bodySegIDTWT2D& s):
	o_data(s.o_data),o_widthStep(s.o_widthStep),subbands(s.subbands),widthSteps(s.widthSteps),maxSize(s.maxSize),w(s.w),level(s.level){
		third_buffer=NULL;	
		row_buffer=NULL;	
		col_buffer=NULL;
		maxWidth = 0;
		maxHeight = 0;
	}

	/*!
\~czech
Destruktor

\~english
Destructor

*/
	~bodySegIDTWT2D();

	/*!
\~czech
Funkce pro poteby paralelnho cyklu parallel_reduce

\~english
Function demanded by the parallel_reduce loop
*/
	void join(const bodySegIDTWT2D& s){}
};
#endif