/*
 * cppTypes.h - pro paraleln vpoet dopen a zptn expanzivn DTWT po segmentech, separabilnm pstupem algoritmem segDTWT2D
 * Autor: Zdenk Pra, UTKO, FEKT, VUT v Brn, Purkyova 118, Brno
 * Program vznikl za podpory internho grantu FEKT-J-10-8 - Optimln algoritmy paralelnho vpotu waveletov transformace rozshlch obrazovch dat
 * Extern knihovny: Intel Threading Building Blocks 3.0
 */


#ifndef CppTypes_H
#define CppTypes_H

#include <fstream>
#include <iostream>
#include <sstream>
#include <string>
#include <malloc.h>
#include <algorithm>
using namespace std;



class Exception : public exception
{
public:
Exception(string m="nastala_chyba") : msg(m) {}
~Exception() throw() {}
const char* what() const throw() { return msg.c_str(); }

private:
string msg;
};

/*! \~czech
Tda reprezentujc separabiln waveletov filtry.
 \~english
Class reprezents wavelet filters.

*/
class separableWavelet{
private:
	// analyzujc doln propust
	float* lo_A;
	// analyzujc horn propust
	float* hi_A;
	// dlka doln propusti
	int length_lo_A;
	// dlka horn propusti
	int length_hi_A;

	// syntetizujc doln propust
	float* lo_R;
	// syntetizujc horn propust
	float* hi_R;
	// dlka doln propusti
	int length_lo_R;
	// dlka horn propusti
	int length_hi_R;

	// nzev waveletu, bude pod tmto nzvem ukldn
string name;

	// otoen verze filtr
	float* lo_A_reversed;
	float* hi_A_reversed;
	float* lo_R_reversed;
	float* hi_R_reversed;
public:

/*!
\~czech
Metoda prohled soubor s nzvem \e file  pro definici vlnkovch filtr s nzvem \e name. Zznam v soubodu file mus mt tento formt:
<pre>
Name = db2
length(LO_A) = 4
LO_A = [ -0.1294095226      0.224143868     0.8365163037     0.4829629131 ];
length(HI_A) = 4
HI_A = [ -0.4829629131     0.8365163037     -0.224143868    -0.1294095226 ];
length(LO_R) = 4
LO_R = [ 0.4829629131     0.8365163037      0.224143868    -0.1294095226 ];
length(HI_R) = 4
HI_R = [ -0.1294095226     -0.224143868     0.8365163037    -0.4829629131 ];
</pre>
a poloka Name mus bt v rmci souboru jedinen.
\param[in] name - nzev vlnky
\param[in] file - nzev souboru k prohledn
\return \e int men ne nula zna chybu pi ten souboru
\exception <Exception> {Vjimka je vyvolna, pokud nen nalezen soubor nebo nzev vlnky}
\sa Exception
\sa save
\sa print

\~english
The method searches a file named \e file for wavelet defined by the name \e name. The file entrny need to be in the following form:
<pre>
Name = db2
length(LO_A) = 4
LO_A = [ -0.1294095226      0.224143868     0.8365163037     0.4829629131 ];
length(HI_A) = 4
HI_A = [ -0.4829629131     0.8365163037     -0.224143868    -0.1294095226 ];
length(LO_R) = 4
LO_R = [ 0.4829629131     0.8365163037      0.224143868    -0.1294095226 ];
length(HI_R) = 4
HI_R = [ -0.1294095226     -0.224143868     0.8365163037    -0.4829629131 ];
</pre>
and the entry \e Name need to be unique.
\param[in] name - wavelet name
\param[in] file - name of the file to be searched
\return \e int if less then zero, error occured while reading the file
\exception <Exception> {The exception is thrown if the file or the wavelet were not found}
\sa Exception
\sa save
\sa print
*/
int load(string name, const char* file = "wavelets.dat"){
		std::ifstream ifs( file,ifstream::in );
		

		if(!ifs.is_open()){ throw Exception("File not found or is damaged.");  } 
		string line;
	
		while(ifs.good()){
		std::getline(ifs,line);
		size_t found = line.find(name);
		if(found!=string::npos){
			string temp;
			std::cout<<"Wavelet "<<name <<" found!! Loading...";

			
			if(lo_A != NULL && hi_A != NULL && hi_R != NULL && lo_R != NULL) clear(); 


			temp = line.erase(0,line.find('=')+1);
			 remove(temp.begin(),temp.end(),' ');
		
			// this->name = temp;

			 

			 // dlka 1. filtru
			 std::getline(ifs,line);
			 temp = line.erase(0,line.find('=')+1);
			stringstream*  ss = new stringstream(temp);	
			if((*ss >> this->length_lo_A).fail()){
			ifs.close();
			delete ss;
			return -2;
			}
			
			// hodnoty 1. filtru
			std::getline(ifs,line);
			temp = line.erase(0,line.find('[')+1);
			temp = temp.erase(temp.find(']'),temp.length());
			delete ss;
			ss = new stringstream(temp,stringstream::in|stringstream::out);
			
			this->lo_A = (float*) _aligned_malloc(length_lo_A*sizeof(float),ALIGN_TO_BYTES);

			for(int i =0;i<length_lo_A;i++)
				*ss >> *(lo_A+i) ;


			 // dlka 2. filtru
			 std::getline(ifs,line);
			 temp = line.erase(0,line.find('=')+1);
			 delete ss;
			ss = new stringstream(temp);	
			if((*ss >> this->length_hi_A).fail()){
			ifs.close();
			return -2;
			}
			
			// hodnoty 2. filtru
			std::getline(ifs,line);
			temp = line.erase(0,line.find('[')+1);
			temp = temp.erase(temp.find(']'),temp.length());
			delete ss;
			ss = new stringstream(temp,stringstream::in|stringstream::out);
			
			this->hi_A = (float*) _aligned_malloc(length_hi_A*sizeof(float),ALIGN_TO_BYTES);

			for(int i =0;i<length_hi_A;i++)
				*ss >> *(hi_A+i) ;

			 // dlka 3. filtru
			 std::getline(ifs,line);
			 temp = line.erase(0,line.find('=')+1);
			 delete ss;
			 ss = new stringstream(temp);	
			if((*ss >> this->length_lo_R).fail()){
			ifs.close();
			return -2;
			}
			
			// hodnoty 3. filtru
			std::getline(ifs,line);
			temp = line.erase(0,line.find('[')+1);
			temp = temp.erase(temp.find(']'),temp.length());
			delete ss;
			ss = new stringstream(temp,stringstream::in|stringstream::out);
			
			this->lo_R = (float*) _aligned_malloc(length_lo_R*sizeof(float),ALIGN_TO_BYTES);

			for(int i =0;i<length_lo_R;i++)
				*ss >> *(lo_R+i) ;

			// dlka 4. filtru
			 std::getline(ifs,line);
			 temp = line.erase(0,line.find('=')+1);
			 delete ss;
			ss = new stringstream(temp);	
			if((*ss >> this->length_hi_R).fail()){
			ifs.close();
			return -2;
			}
			
			// hodnoty 4. filtru
			std::getline(ifs,line);
			temp = line.erase(0,line.find('[')+1);
			temp = temp.erase(temp.find(']'),temp.length());
			delete ss;
			ss = new stringstream(temp,stringstream::in|stringstream::out);
			
			this->hi_R = (float*) _aligned_malloc(length_hi_R*sizeof(float),ALIGN_TO_BYTES);

			for(int i =0;i<length_hi_R;i++)
				*ss >> *(hi_R+i) ;
			

			std::cout<<"Loaded."<<std::endl;
			delete ss;
			ifs.close();

			reverseCopy();
			return 0;
		}
		
		}

		ifs.close();

		std::cout<<"Wavelet "<< name <<" not found."<<std::endl; 
		throw Exception("Wavelet not found!");
		return -1;
	}


/*!
~czech
Metoda ulo nzev a koeficienty filtr do souboru \e file pokud tam ji nen poloka se stejnm jmnem. Formt zznamu je stejn jako u funkce \e load

\param[in] - file nzev souboru pro uloen filtr
\return int men ne nula zna chybu pi ten souboru
\sa load
\sa print

~english
The method saves name and filter coefficients to file \file if the same name is not already there. The entry format is the same as described in method \e load

\param[in] - file the name of a file filter kefiient willbe added to
\return  int if less then zero, error occured while reading the file
\sa load
\sa print
*/
int save(const char* file = "wavelets.dat") const{
	
	std::ifstream ifs( file,ifstream::in );
		string line;

		while(ifs.good()){
		std::getline(ifs,line);
		size_t found = line.find(name);
		if(found!=string::npos){
			std::cout<<"This wavelet is already in file"<<std::endl;
			ifs.close();
			return -2;
		}
		//std::cout<<line<<std::endl;
		}
		ifs.close();

		std::ofstream ofs(file,std::ios_base::app);
		if(!ofs.is_open()) return -1;


		ofs.seekp(0,std::ios_base::beg);
	
		ofs << "\n";
		ofs << "Name = ";
		ofs.write(name.data(),name.length());
		ofs << "\n";

		ofs << "length(LO_A) = " << length_lo_A << "\n";

		ofs << "LO_A = [ ";
		ofs.setf(0,ios::floatfield);
		ofs.precision(10);
		for(int i=0;i<length_lo_A;i++)
			ofs << lo_A[i] << " "; 

		ofs << "]; ";
		ofs << "\n";

		ofs << "length(HI_A) = " << length_hi_A<< "\n";

		ofs << "HI_A = [ ";
		ofs.setf(0,ios::floatfield);
		ofs.precision(10);
		for(int i=0;i<length_hi_A;i++)
			ofs << hi_A[i] << " "; 

		ofs << "]; ";
		ofs << "\n";

		ofs << "length(LO_R) = " << length_lo_R<< "\n";

		ofs << "LO_R = [ ";
		ofs.setf(0,ios::floatfield);
		ofs.precision(10);
		for(int i=0;i<length_lo_R;i++)
			ofs << lo_R[i] << " "; 

		ofs << "]; ";
		ofs << "\n";

		ofs << "length(HI_R) = " << length_hi_R<< "\n";

		ofs << "HI_R = [ ";
		ofs.setf(0,ios::floatfield);
		ofs.precision(10);
		for(int i=0;i<length_hi_R;i++)
			ofs << hi_R[i] << " "; 

		ofs << "]; ";
		ofs << "\n";

		


		ofs.flush();
		ofs.close();
		return 0;
	}

/*!
\~czech
Metoda vytiskne nzev akoeficienty filtr pro aktuln vlnku.

\~english
The method prints name and filter coefficients associated with the wavelet.

*/
void print() const{
		std::cout << "Name = "; 
		std::cout.write(name.data(),name.length()-1);
			
		std::cout << "\n";


		std::cout << "length(LO_A) = " << length_lo_A << "\n";

		std::cout << "LO_A = [ ";
		std::cout.setf(0,ios::floatfield);
		std::cout.precision(10);
		for(int i=0;i<length_lo_A;i++)
			std::cout << lo_A[i] << " "; 

		std::cout << "]; ";
		std::cout << "\n";

		std::cout << "length(HI_A) = " << length_hi_A<< "\n";

		std::cout << "HI_A = [ ";
		std::cout.setf(0,ios::floatfield);
		std::cout.precision(10);
		for(int i=0;i<length_hi_A;i++)
			std::cout << hi_A[i] << " "; 

		std::cout << "]; ";
		std::cout << "\n";

		std::cout << "length(LO_R) = " << length_lo_R<< "\n";

		std::cout << "LO_R = [ ";
		std::cout.setf(0,ios::floatfield);
		std::cout.precision(10);
		for(int i=0;i<length_lo_R;i++)
			std::cout << lo_R[i] << " "; 

		std::cout << "]; ";
		std::cout << "\n";

		std::cout << "length(HI_R) = " << length_hi_R<< "\n";

		std::cout << "HI_R = [ ";
		std::cout.setf(0,ios::floatfield);
		std::cout.precision(10);
		for(int i=0;i<length_hi_R;i++)
			std::cout << hi_R[i] << " "; 

		std::cout << "]; \n";
	
		
	}


/*!

\~czech
Metoda uvoluje vechny buffery.

\~english
The method frees all buffers

*/
void clear(){
		_aligned_free(lo_A);
		_aligned_free(hi_A);
		_aligned_free(lo_R);
		_aligned_free(hi_R);
		_aligned_free(lo_A_reversed);
		_aligned_free(hi_A_reversed);
		_aligned_free(lo_R_reversed);
		_aligned_free(hi_R_reversed);
	}



	//separableWavelet():length_lo_A(0),length_hi_A(0),length_lo_R(0),length_hi_R(0),lo_A(NULL),hi_A(NULL),lo_R(NULL),hi_R(NULL),name(""){}

/*!
\~czech
Konstruktor
\param[in] - nzev poadovan vlnky 
\param[in] - textov soubor, kter se m prohledvat
\~english
Constructor
\param[in] - name of the wavelet 
\param[in] - text file to be searched
*/
separableWavelet(string name="default",const char* file = "wavelets.dat") :length_lo_A(0),length_hi_A(0),length_lo_R(0),length_hi_R(0),lo_A(NULL),hi_A(NULL),lo_R(NULL),hi_R(NULL),name(name) {
	try{
		int i =load(name,file);
		if(i<0) throw;
	}catch(...){
      std::cout << "File " << file << " was not found or is damaged." << std::endl; 
	 throw;
	}
	}

/*!
\~czech
Konstruktor
\param[in] - *lo_A ukazatel na pole koeficient analyzujc doln propusti DP
\param[in] - *hi_A ukazatel na pole koeficient analyzujc horn propusti HP
\param[in] - length_lo_A poet koeficient anylyzujc DP
\param[in] - length_hi_A poet koeficient anylyzujc HP
\~english
Constructor
\param[in] - *lo_A pointer to the array of coefficients of the low pass filter LP
\param[in] - *hi_A pointer to the array of coefficients of the high pass filter HP
\param[in] - length_lo_A number of coefficients of LP
\param[in] - length_hi_A number of coefficients of HP
*/
	separableWavelet(float* lo_A,float* hi_A,int length_lo_A,int length_hi_A){
		initialize(lo_A,hi_A,length_lo_A);
		this->length_hi_A = length_hi_A;
	}
/*!
\~czech
Konstruktor, pedpold se, e oba filtry maj stejn poet koeficient
\param[in] - *lo_A ukazatel na pole koeficient analyzujc doln propusti DP
\param[in] - *hi_A ukazatel na pole koeficient analyzujc horn propusti HP
\param[in] - length_lo_A poet koeficient anylyzujc DP
\~english
Constructor, the same number of coefficients is assumed for both filters
\param[in] - *lo_A pointer to the array of coefficients of the low pass filter LP
\param[in] - *hi_A pointer to the array of coefficients of the high pass filter HP
\param[in] - length_lo_A number of coefficients of LP
*/
	separableWavelet(float* lo_A,float* hi_A,int length_lo_A){
		initialize(lo_A,hi_A,length_lo_A);
	}
/*!
\~czech
Konstruktor
\param[in] - *lo_A ukazatel na pole koeficient analyzujc doln propusti DP
\param[in] - *hi_A ukazatel na pole koeficient analyzujc horn propusti HP
\param[in] - length_lo_A poet koeficient kadho z filtr
\param[in] - name nzev vlnky
\~english
Constructor
\param[in] - *lo_A pointer to the array of coefficients of the low pass filter LP
\param[in] - *hi_A pointer to the array of coefficients of the high pass filter HP
\param[in] - length_lo_A number of coefficients of both filters
\param[in] - name wavelet name
*/
	separableWavelet(float* lo_A,float* hi_A,int length_lo_A,string name){
		initialize(lo_A,hi_A,length_lo_A);
		this->name = name;
	}
/*!
\~czech
Konstruktor
\param[in] - *lo_A ukazatel na pole koeficient analyzujc doln propusti DP
\param[in] - *hi_A ukazatel na pole koeficient analyzujc horn propusti HP
\param[in] - length_lo_A poet koeficient anylyzujc DP
\param[in] - length_hi_A poet koeficient anylyzujc HP
\param[in] - name nzev vlnky
\~english
Constructor
\param[in] - *lo_A pointer to the array of coefficients of the low pass filter LP
\param[in] - *hi_A pointer to the array of coefficients of the high pass filter HP
\param[in] - length_lo_A number of coefficients of LP
\param[in] - length_hi_A number of coefficients of HP
\param[in] - name wavelet name
*/
	separableWavelet(float* lo_A,float* hi_A,int length_lo_A,int length_hi_A,string name){
		initialize(lo_A,hi_A,length_lo_A);
		this->length_hi_A = length_hi_A;
		this->name = name;
	}


/*!
\~czech
Koprovac konstruktor
\param[in] - &old reference na pvodn objekt

\~english
Copy Constructor
\param[in] - &old reference to the former object
*/
	separableWavelet(const separableWavelet& old){
	this->name = old.name;
	this->length_lo_A = length_lo_A;
	this->length_hi_A = length_hi_A;
	this->length_hi_R = length_hi_R;
	this->length_lo_R = length_lo_R;


	this->lo_A = (float*) _aligned_malloc(length_lo_A*sizeof(float),ALIGN_TO_BYTES);
	std::copy(old.lo_A,old.lo_A+length_lo_A,this->lo_A);

	this->hi_A = (float*) _aligned_malloc(length_hi_A*sizeof(float),ALIGN_TO_BYTES);
	std::copy(old.hi_A,old.hi_A+length_hi_A,this->hi_A);

	this->lo_R = (float*) _aligned_malloc(length_lo_R*sizeof(float),ALIGN_TO_BYTES);
	std::copy(old.lo_R,old.lo_R+length_lo_R,this->lo_R);

	this->hi_R = (float*) _aligned_malloc(length_hi_R*sizeof(float),ALIGN_TO_BYTES);
	std::copy(old.hi_R,old.hi_R+length_hi_R,this->hi_R);
	}

	
	int getMaxFilterLengthRows() const{
		return max(length_lo_A,length_hi_A);
	}

	int getMaxFilterLengthCols() const{
		return max(length_lo_A,length_hi_A);
	}

	~separableWavelet(){
		clear();
	}

	
	int getRightZeros(int no) const {
		 int actual = getAnalysingFilterLength(no);
		 int maximal = getMaxFilterLengthCols();
		 int zeros = (maximal - actual)/2;

		 return (odd(zeros)?zeros-1:zeros);
	 }

	 int getLeftZeros(int no)const {
	 	 int actual = getAnalysingFilterLength(no);
		 int maximal = getMaxFilterLengthCols();
		 int zeros = (maximal - actual)/2;

		 return (odd(zeros)?zeros+1:zeros);
	 }

	int getRightZerosSyn(int no) const {
		 int actual = getSyntetizingFilterLength(no);
		 int maximal = getMaxFilterLengthCols();
		 int zeros = (maximal - actual)/2;

		 return (odd(zeros)?zeros-1:zeros);
	 }

	 int getLeftZerosSyn(int no)const {
	 	 int actual = getSyntetizingFilterLength(no);
		 int maximal = getMaxFilterLengthCols();
		 int zeros = (maximal - actual)/2;

		 return (odd(zeros)?zeros+1:zeros);
	 }


	 int getTopZeros(int no) const{
		 int actual = getAnalysingFilterLength(no);
		 int maximal = getMaxFilterLengthRows();
		 int zeros = (maximal - actual)/2;

		 return (odd(zeros)?zeros+1:zeros);
	 }

	 int getBottomZeros(int no) const{
		int actual = getAnalysingFilterLength(no);
		 int maximal = getMaxFilterLengthRows();
		 int zeros = (maximal - actual)/2;

		 return (odd(zeros)?zeros-1:zeros);
	 }
	
	float* getAnalysingFilter(int no) const{
		switch(no){
		case 0:
		 return lo_A_reversed;
		break;
		case 1:
		 return hi_A_reversed;
		break;
		default:
		 return 0;
		}
	}

	int getAnalysingFilterLength(int no) const{
		switch(no){
		case 0:
			return length_lo_A;
		break;
		case 1:
			return length_hi_A;
		break;
		default:
		 return 0;
		}
	}

	float* getSyntetizingFilter(int no) const{
		switch(no){
		case 0:
		 return lo_R_reversed;
		break;
		case 1:
		 return hi_R_reversed;
		break;
		default:
		 return 0;
		}
	}

	int getSyntetizingFilterLength(int no) const{
		switch(no){
		case 0:
		 return length_lo_R;
		break;
		case 1:
		 return length_hi_R;
		break;
		default:
		 return 0;
		}
	}
	
	
	
	float* getFilter(int no) const{
		switch(no){
		case 0:
		 return lo_A_reversed;
		break;
		case 1:
		 return hi_A_reversed;
		break;
		case 2:
		 return lo_R_reversed;
		break;
		case 3:
		 return hi_R_reversed;
		break;
		default:
		 return 0;
		}

	}

	int getFilterLength(int no) const{
		switch(no){
		case 0:
		 return length_lo_A;
		break;
		case 1:
		 return length_hi_A;
		break;
		case 2:
		 return length_lo_R;
		break;
		case 3:
		 return length_hi_R;
		break;
		}
return -1;
	}

	int getLength() const{
	 return length_lo_A;
	}

private:

void initialize(float* lo_A,float* hi_A,int length_lo_A){
		this->name = "default";
		this->length_lo_A = length_lo_A;
		this->length_hi_A = length_lo_A;
	
		this->lo_A = (float*) _aligned_malloc(length_lo_A*sizeof(float),ALIGN_TO_BYTES);
		std::copy(lo_A,lo_A+length_lo_A,this->lo_A);

		this->hi_A = (float*) _aligned_malloc(length_hi_A*sizeof(float),ALIGN_TO_BYTES);
		std::copy(hi_A,hi_A+length_hi_A,this->hi_A);

		length_hi_R = length_lo_A;
		length_lo_R = length_hi_A;

		this->lo_R = (float*) _aligned_malloc(length_lo_R*sizeof(float),ALIGN_TO_BYTES);
	
		for(int i=0;i<length_lo_R;i++){
			this->lo_R[i] = this->hi_A[i];
			if(i%2==1)
				this->lo_R[i] = -this->lo_R[i];
		}

		this->hi_R = (float*) _aligned_malloc(length_hi_R*sizeof(float),ALIGN_TO_BYTES);

		for(int i=0;i<length_hi_R;i++){
			this->hi_R[i] = this->lo_A[i];
			if(i%2!=1)
				this->hi_R[i] = -this->hi_R[i];
		}

		reverseCopy();
	}

void reverseCopy(){
	this->lo_A_reversed = (float*) _aligned_malloc(length_lo_A*sizeof(float),ALIGN_TO_BYTES);
	this->hi_A_reversed = (float*) _aligned_malloc(length_hi_A*sizeof(float),ALIGN_TO_BYTES);
	this->lo_R_reversed = (float*) _aligned_malloc(length_lo_R*sizeof(float),ALIGN_TO_BYTES);
	this->hi_R_reversed = (float*) _aligned_malloc(length_hi_R*sizeof(float),ALIGN_TO_BYTES);

	std::reverse_copy(lo_A,lo_A + length_lo_A,lo_A_reversed);
	std::reverse_copy(hi_A,hi_A + length_hi_A,hi_A_reversed);
	std::reverse_copy(lo_R,lo_R + length_lo_R,lo_R_reversed);
	std::reverse_copy(hi_R,hi_R + length_hi_R,hi_R_reversed);
	
	}



};


#endif