/*
 * memoryAllocation.h - pro paraleln vpoet dopen a zptn expanzivn DTWT po segmentech, separabilnm pstupem algoritmem segDTWT2D
 * Autor: Zdenk Pra, UTKO, FEKT, VUT v Brn, Purkyova 118, Brno
 * Program vznikl za podpory internho grantu FEKT-J-10-8 - Optimln algoritmy paralelnho vpotu waveletov transformace rozshlch obrazovch dat
 * Extern knihovny: Intel Threading Building Blocks 3.0
 */
#ifndef memoryAllocation_H
#define memoryAllocation_H

#include "segDTWT.h"
#include "malloc.h"
#include "tbb/scalable_allocator.h"

#ifdef _DEBUG
    #pragma comment(lib, "tbbmalloc_debug.lib")
#else
    #pragma comment(lib, "tbbmalloc.lib")
#endif


/*!
\~czech
Funkce alokuje jednokanlov obraz jako blok pamti o  *widthStep x height x sizeof(float) bytech. Zatky vech dk jsou zarovnny na adresy dliteln 16ti.

\param[in] width - poadovan vka obrazu v pixelech
\param[in] height - poadovan ka obrazu v pixelech
\param[in,out] *widthStep - vzdlenost mezi jednotlivmi dky v bytech
\return ukazatel na zatek alokovanho bloku
\sa releaseImage
\sa allocateSubbands
\sa releaseSubbands

\~english
The function allocates an image as a *widthStep x height x sizeof(float) byte block of memory. In the memory the beginning of each row is 16 byte aligned. 

\param[in] width - the width of image in pixels
\param[in] height - the height of image in pixels
\param[in,out] *widthStep - the number of bytes between two conscutive rows
\return pointer to the beginning of allocated block of memory
\sa releaseImage
\sa allocateSubbands
\sa releaseSubbands
*/
float* allocateImage_32f_C1(int width, int height, int* widthStep);


/*!
\~czech
Funkce uvoluje pam alokovanou funkc allocateImage_32f_C1.

\param[in] *image - ukazatel na blok pamti k uvolnn
\sa allocateImage_32f_C1
\sa allocateSubbands
\sa releaseSubbands

\~english
The function frees the memory allocated by the function allocateImage_32f_C1

\param[in] *image - pointer to memoty to be freed
\sa allocateImage_32f_C1
\sa allocateSubbands
\sa releaseSubbands
*/
void releaseImage(void* image);

/*!
\~czech
Funkce alokuje (3xlevels+1) blok pamti pro uloen jednotlivch vstupnch subpsem. Ukazatele na subpsma jsou za sebou uloeny v poad subbands[0]=HL1,[1]=HH1,[2]=LH1,[3]=HL2,[4]=HH2,[5]=LH2 .... [last]=LL. Rozmry subpsem jsou: \f[ n_\mathrm{x}(s,m,j) = \mathrm{floor}( 2^{-j}s_\mathrm{x} + (1-2^{-j}(m-1)) \f] a 
							\f[ n_\mathrm{y}(s,m,j) = \mathrm{floor}( 2^{-j}s_\mathrm{y} + (1-2^{-j}(m-1)), \f] piem
							- \f$ s,s_x,s_y \f$ jsou rozmry vstupnho obrazu,
							- \f$ j \f$ stupe dekompozice,
							- \f$ m \f$ dlka pouitch filtr.

\param[in,out] *subbands[] - pole uchovvajc ukazatele na jednotliv sublsma
\param[in,out] widthSteps[] - pole uchovvajc vzdlenosti mezi dky subpsem v bytech jednotlivch subpsem
\param[in] level - rove dekompozice
\param[in] filter_length_L - dlka analyzujc doln propusti
\param[in] filter_length_H - dlka analyzujc horn propusti
\param[in] size - rozmry pvodnho obrazu v pixelech
\return stavov kd WRESULT
\sa releaseImage
\sa releaseSubbands
\sa segDTWTfwd_32f_C1
\sa segDTWTinv_32f_C1

\~english
The function allocates (3xlevels+1) blocks of memory to hold individual subbands. The order of the subbands is the following: subbands[0]=HL1,[1]=HH1,[2]=LH1,[3]=HL2,[4]=HH2,[5]=LH2 .... [last]=LL. The dimensions of the subbands are: \f[ n_\mathrm{x}(s,m,j) = \mathrm{floor}( 2^{-j}s_\mathrm{x} + (1-2^{-j}(m-1)) \f] and 
							\f[ n_\mathrm{y}(s,m,j) = \mathrm{floor}( 2^{-j}s_\mathrm{y} + (1-2^{-j}(m-1)), \f] piem
							- \f$ s,s_x,s_y \f$ are dimensions of the input image in pixels,
							- \f$ j \f$ denotes the depth of decomposition,
							- \f$ m \f$ is the length of used filters.

\param[in,out] *subbands[] - array holding ponters to the individual subbands
\param[in,out] widthSteps[] - array holding the distance between two consecutive rows in bytes in individual subbands
\param[in] level - the depth of decomposition
\param[in] filter_length_L - the length of analysing lowpass filter
\param[in] filter_length_H - the length of analysing highpass filter
\param[in] size - the dimensions of input images in pixels
\return stavov kd WRESULT
\sa releaseImage
\sa releaseSubbands
\sa segDTWTfwd_32f_C1
\sa segDTWTinv_32f_C1
*/
WRESULT allocateSubbands(float* subbands[],int widthSteps[],int level,int filter_length_L,int filter_length_H,Size size);

/*!
\~czech
Funkce uvoluje pam alokovanou pomoc funkce allocateSubbands.

\param[in] * subbands[] - pole uchovvajc ukazatele na jednotliv sublsma
\param[in] level - hloubka dekompozice

\return stavov kd WRESULT
\sa allocateSubbands

\~english
Funkce frees memory allocated by the function allocateSubbands.

\param[in] * subbands[] - array holding ponters to the individual subbands
\param[in] level - the depth of decomposition

\return stavov kd WRESULT
\sa allocateSubbands
*/
WRESULT releaseSubbands(float* subbands[], int level);

#endif